"""This type is used to mark values returned from store (arrays of bytes).

NOTE: Currently, this type is only used in the view_client and RPC to be able to transparently
pretty-serialize the bytes arrays as base64-encoded strings (see `serialize.rs`)."""

from pydantic import BaseModel
from pydantic import RootModel


class StoreValue(RootModel[str]):
    pass




"""Describes the error for validating a list of actions."""

from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.near_gas import NearGas
from near_jsonrpc_models.near_token import NearToken
from near_jsonrpc_models.public_key import PublicKey
from near_jsonrpc_models.strict_model import StrictBaseModel
from pydantic import BaseModel
from pydantic import RootModel
from pydantic import conint
from typing import Literal
from typing import Union


"""The delete action must be a final action in transaction"""
class ActionsValidationErrorDeleteActionMustBeFinal(RootModel[Literal['DeleteActionMustBeFinal']]):
    pass

class ActionsValidationErrorTotalPrepaidGasExceededPayload(BaseModel):
    limit: NearGas
    total_prepaid_gas: NearGas

class ActionsValidationErrorTotalPrepaidGasExceeded(StrictBaseModel):
    """The total prepaid gas (for all given actions) exceeded the limit."""
    TotalPrepaidGasExceeded: ActionsValidationErrorTotalPrepaidGasExceededPayload

class ActionsValidationErrorTotalNumberOfActionsExceededPayload(BaseModel):
    limit: conint(ge=0, le=18446744073709551615)
    total_number_of_actions: conint(ge=0, le=18446744073709551615)

class ActionsValidationErrorTotalNumberOfActionsExceeded(StrictBaseModel):
    """The number of actions exceeded the given limit."""
    TotalNumberOfActionsExceeded: ActionsValidationErrorTotalNumberOfActionsExceededPayload

class ActionsValidationErrorAddKeyMethodNamesNumberOfBytesExceededPayload(BaseModel):
    limit: conint(ge=0, le=18446744073709551615)
    total_number_of_bytes: conint(ge=0, le=18446744073709551615)

class ActionsValidationErrorAddKeyMethodNamesNumberOfBytesExceeded(StrictBaseModel):
    """The total number of bytes of the method names exceeded the limit in a Add Key action."""
    AddKeyMethodNamesNumberOfBytesExceeded: ActionsValidationErrorAddKeyMethodNamesNumberOfBytesExceededPayload

class ActionsValidationErrorAddKeyMethodNameLengthExceededPayload(BaseModel):
    length: conint(ge=0, le=18446744073709551615)
    limit: conint(ge=0, le=18446744073709551615)

class ActionsValidationErrorAddKeyMethodNameLengthExceeded(StrictBaseModel):
    """The length of some method name exceeded the limit in a Add Key action."""
    AddKeyMethodNameLengthExceeded: ActionsValidationErrorAddKeyMethodNameLengthExceededPayload

"""Integer overflow during a compute."""
class ActionsValidationErrorIntegerOverflow(RootModel[Literal['IntegerOverflow']]):
    pass

class ActionsValidationErrorInvalidAccountIdPayload(BaseModel):
    account_id: str

class ActionsValidationErrorInvalidAccountId(StrictBaseModel):
    """Invalid account ID."""
    InvalidAccountId: ActionsValidationErrorInvalidAccountIdPayload

class ActionsValidationErrorContractSizeExceededPayload(BaseModel):
    limit: conint(ge=0, le=18446744073709551615)
    size: conint(ge=0, le=18446744073709551615)

class ActionsValidationErrorContractSizeExceeded(StrictBaseModel):
    """The size of the contract code exceeded the limit in a DeployContract action."""
    ContractSizeExceeded: ActionsValidationErrorContractSizeExceededPayload

class ActionsValidationErrorFunctionCallMethodNameLengthExceededPayload(BaseModel):
    length: conint(ge=0, le=18446744073709551615)
    limit: conint(ge=0, le=18446744073709551615)

class ActionsValidationErrorFunctionCallMethodNameLengthExceeded(StrictBaseModel):
    """The length of the method name exceeded the limit in a Function Call action."""
    FunctionCallMethodNameLengthExceeded: ActionsValidationErrorFunctionCallMethodNameLengthExceededPayload

class ActionsValidationErrorFunctionCallArgumentsLengthExceededPayload(BaseModel):
    length: conint(ge=0, le=18446744073709551615)
    limit: conint(ge=0, le=18446744073709551615)

class ActionsValidationErrorFunctionCallArgumentsLengthExceeded(StrictBaseModel):
    """The length of the arguments exceeded the limit in a Function Call action."""
    FunctionCallArgumentsLengthExceeded: ActionsValidationErrorFunctionCallArgumentsLengthExceededPayload

class ActionsValidationErrorUnsuitableStakingKeyPayload(BaseModel):
    public_key: PublicKey

class ActionsValidationErrorUnsuitableStakingKey(StrictBaseModel):
    """An attempt to stake with a public key that is not convertible to ristretto."""
    UnsuitableStakingKey: ActionsValidationErrorUnsuitableStakingKeyPayload

"""The attached amount of gas in a FunctionCall action has to be a positive number."""
class ActionsValidationErrorFunctionCallZeroAttachedGas(RootModel[Literal['FunctionCallZeroAttachedGas']]):
    pass

"""There should be the only one DelegateAction"""
class ActionsValidationErrorDelegateActionMustBeOnlyOne(RootModel[Literal['DelegateActionMustBeOnlyOne']]):
    pass

class ActionsValidationErrorUnsupportedProtocolFeaturePayload(BaseModel):
    protocol_feature: str
    version: conint(ge=0, le=4294967295)

class ActionsValidationErrorUnsupportedProtocolFeature(StrictBaseModel):
    """The transaction includes a feature that the current protocol version
does not support.

Note: we stringify the protocol feature name instead of using
`ProtocolFeature` here because we don't want to leak the internals of
that type into observable borsh serialization."""
    UnsupportedProtocolFeature: ActionsValidationErrorUnsupportedProtocolFeaturePayload

class ActionsValidationErrorInvalidDeterministicStateInitReceiverPayload(BaseModel):
    derived_id: AccountId
    receiver_id: AccountId

class ActionsValidationErrorInvalidDeterministicStateInitReceiver(StrictBaseModel):
    InvalidDeterministicStateInitReceiver: ActionsValidationErrorInvalidDeterministicStateInitReceiverPayload

class ActionsValidationErrorDeterministicStateInitKeyLengthExceededPayload(BaseModel):
    length: conint(ge=0, le=18446744073709551615)
    limit: conint(ge=0, le=18446744073709551615)

class ActionsValidationErrorDeterministicStateInitKeyLengthExceeded(StrictBaseModel):
    DeterministicStateInitKeyLengthExceeded: ActionsValidationErrorDeterministicStateInitKeyLengthExceededPayload

class ActionsValidationErrorDeterministicStateInitValueLengthExceededPayload(BaseModel):
    length: conint(ge=0, le=18446744073709551615)
    limit: conint(ge=0, le=18446744073709551615)

class ActionsValidationErrorDeterministicStateInitValueLengthExceeded(StrictBaseModel):
    DeterministicStateInitValueLengthExceeded: ActionsValidationErrorDeterministicStateInitValueLengthExceededPayload

class ActionsValidationErrorGasKeyInvalidNumNoncesPayload(BaseModel):
    limit: conint(ge=0, le=65535)
    requested_nonces: conint(ge=0, le=65535)

class ActionsValidationErrorGasKeyInvalidNumNonces(StrictBaseModel):
    GasKeyInvalidNumNonces: ActionsValidationErrorGasKeyInvalidNumNoncesPayload

class ActionsValidationErrorAddGasKeyWithNonZeroBalancePayload(BaseModel):
    balance: NearToken

class ActionsValidationErrorAddGasKeyWithNonZeroBalance(StrictBaseModel):
    AddGasKeyWithNonZeroBalance: ActionsValidationErrorAddGasKeyWithNonZeroBalancePayload

"""Gas keys with FunctionCall permission cannot have an allowance set."""
class ActionsValidationErrorGasKeyFunctionCallAllowanceNotAllowed(RootModel[Literal['GasKeyFunctionCallAllowanceNotAllowed']]):
    pass

class ActionsValidationError(RootModel[Union[ActionsValidationErrorDeleteActionMustBeFinal, ActionsValidationErrorTotalPrepaidGasExceeded, ActionsValidationErrorTotalNumberOfActionsExceeded, ActionsValidationErrorAddKeyMethodNamesNumberOfBytesExceeded, ActionsValidationErrorAddKeyMethodNameLengthExceeded, ActionsValidationErrorIntegerOverflow, ActionsValidationErrorInvalidAccountId, ActionsValidationErrorContractSizeExceeded, ActionsValidationErrorFunctionCallMethodNameLengthExceeded, ActionsValidationErrorFunctionCallArgumentsLengthExceeded, ActionsValidationErrorUnsuitableStakingKey, ActionsValidationErrorFunctionCallZeroAttachedGas, ActionsValidationErrorDelegateActionMustBeOnlyOne, ActionsValidationErrorUnsupportedProtocolFeature, ActionsValidationErrorInvalidDeterministicStateInitReceiver, ActionsValidationErrorDeterministicStateInitKeyLengthExceeded, ActionsValidationErrorDeterministicStateInitValueLengthExceeded, ActionsValidationErrorGasKeyInvalidNumNonces, ActionsValidationErrorAddGasKeyWithNonZeroBalance, ActionsValidationErrorGasKeyFunctionCallAllowanceNotAllowed]]):
    pass




"""Bitmap which describes which values from the predefined list are being requested.
The nth bit is set to 1 when the nth value from the list is being requested."""

from pydantic import BaseModel
from pydantic import conint
from pydantic import conlist
from typing import List


class BandwidthRequestBitmap(BaseModel):
    data: conlist(conint(ge=0, le=255), min_length=5, max_length=5)



"""Describes information about an access key including the public key."""

from near_jsonrpc_models.access_key_view import AccessKeyView
from near_jsonrpc_models.public_key import PublicKey
from pydantic import BaseModel


class AccessKeyInfoView(BaseModel):
    access_key: AccessKeyView
    public_key: PublicKey



"""URIs for the Chunk Distribution Network feature."""

from pydantic import BaseModel


class ChunkDistributionUris(BaseModel):
    # URI for pulling chunks from the stream.
    get: str = None
    # URI for publishing chunks to the stream.
    set: str = None



"""This type is used to mark function arguments.

NOTE: The main reason for this to exist (except the type-safety) is that the value is
transparently serialized and deserialized as a base64-encoded string when serde is used
(serde_json)."""

from pydantic import BaseModel
from pydantic import RootModel


class FunctionArgs(RootModel[str]):
    pass




from near_jsonrpc_models.chunk_hash import ChunkHash
from near_jsonrpc_models.shard_id import ShardId
from pydantic import BaseModel
from pydantic import RootModel
from typing import Any
from typing import Dict
from typing import Literal
from typing import Union


class RpcChunkErrorInternalErrorInfo(BaseModel):
    error_message: str

class RpcChunkErrorInternalError(BaseModel):
    info: RpcChunkErrorInternalErrorInfo
    name: Literal['INTERNAL_ERROR']

class RpcChunkErrorUnknownBlock(BaseModel):
    info: Dict[str, Any]
    name: Literal['UNKNOWN_BLOCK']

class RpcChunkErrorInvalidShardIdInfo(BaseModel):
    shard_id: ShardId

class RpcChunkErrorInvalidShardId(BaseModel):
    info: RpcChunkErrorInvalidShardIdInfo
    name: Literal['INVALID_SHARD_ID']

class RpcChunkErrorUnknownChunkInfo(BaseModel):
    chunk_hash: ChunkHash

class RpcChunkErrorUnknownChunk(BaseModel):
    info: RpcChunkErrorUnknownChunkInfo
    name: Literal['UNKNOWN_CHUNK']

class RpcChunkError(RootModel[Union[RpcChunkErrorInternalError, RpcChunkErrorUnknownBlock, RpcChunkErrorInvalidShardId, RpcChunkErrorUnknownChunk]]):
    pass




from near_jsonrpc_models.rpc_state_changes_in_block_request import RpcStateChangesInBlockRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForBlockEffects(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['block_effects']
    params: RpcStateChangesInBlockRequest



from near_jsonrpc_models.block_id import BlockId
from near_jsonrpc_models.crypto_hash import CryptoHash
from near_jsonrpc_models.shard_id import ShardId
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class RpcChunkRequestBlockShardId(BaseModel):
    block_id: BlockId
    shard_id: ShardId

class RpcChunkRequestChunkHash(BaseModel):
    chunk_id: CryptoHash

class RpcChunkRequest(RootModel[Union[RpcChunkRequestBlockShardId, RpcChunkRequestChunkHash]]):
    pass




"""Execution outcome of the transaction and all the subsequent receipts.
Could be not finalized yet"""

from near_jsonrpc_models.execution_outcome_with_id_view import ExecutionOutcomeWithIdView
from near_jsonrpc_models.final_execution_status import FinalExecutionStatus
from near_jsonrpc_models.signed_transaction_view import SignedTransactionView
from pydantic import BaseModel
from typing import List


class FinalExecutionOutcomeView(BaseModel):
    # The execution outcome of receipts.
    receipts_outcome: List[ExecutionOutcomeWithIdView]
    # Execution status defined by chain.rs:get_final_transaction_result
    # FinalExecutionStatus::NotStarted - the tx is not converted to the receipt yet
    # FinalExecutionStatus::Started - we have at least 1 receipt, but the first leaf receipt_id (using dfs) hasn't finished the execution
    # FinalExecutionStatus::Failure - the result of the first leaf receipt_id
    # FinalExecutionStatus::SuccessValue - the result of the first leaf receipt_id
    status: FinalExecutionStatus
    # Signed Transaction
    transaction: SignedTransactionView
    # The execution outcome of the signed transaction.
    transaction_outcome: ExecutionOutcomeWithIdView



"""The shard identifier. It may be an arbitrary number - it does not need to be
a number in the range 0..NUM_SHARDS. The shard ids do not need to be
sequential or contiguous.

The shard id is wrapped in a new type to prevent the old pattern of using
indices in range 0..NUM_SHARDS and casting to ShardId. Once the transition
if fully complete it potentially may be simplified to a regular type alias."""

from pydantic import BaseModel
from pydantic import RootModel
from pydantic import conint


class ShardId(RootModel[conint(ge=0, le=18446744073709551615)]):
    pass




from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.block_reference import BlockReference
from near_jsonrpc_models.crypto_hash import CryptoHash
from pydantic import BaseModel
from pydantic import RootModel
from pydantic import conint
from typing import Literal
from typing import Union


class RpcViewStateErrorUnknownBlockInfo(BaseModel):
    block_reference: BlockReference

class RpcViewStateErrorUnknownBlock(BaseModel):
    info: RpcViewStateErrorUnknownBlockInfo
    name: Literal['UNKNOWN_BLOCK']

class RpcViewStateErrorInvalidAccountInfo(BaseModel):
    block_hash: CryptoHash
    block_height: conint(ge=0, le=18446744073709551615)
    requested_account_id: AccountId

class RpcViewStateErrorInvalidAccount(BaseModel):
    info: RpcViewStateErrorInvalidAccountInfo
    name: Literal['INVALID_ACCOUNT']

class RpcViewStateErrorUnknownAccountInfo(BaseModel):
    block_hash: CryptoHash
    block_height: conint(ge=0, le=18446744073709551615)
    requested_account_id: AccountId

class RpcViewStateErrorUnknownAccount(BaseModel):
    info: RpcViewStateErrorUnknownAccountInfo
    name: Literal['UNKNOWN_ACCOUNT']

class RpcViewStateErrorTooLargeContractStateInfo(BaseModel):
    block_hash: CryptoHash
    block_height: conint(ge=0, le=18446744073709551615)
    contract_account_id: AccountId

class RpcViewStateErrorTooLargeContractState(BaseModel):
    info: RpcViewStateErrorTooLargeContractStateInfo
    name: Literal['TOO_LARGE_CONTRACT_STATE']

class RpcViewStateErrorInternalErrorInfo(BaseModel):
    error_message: str

class RpcViewStateErrorInternalError(BaseModel):
    info: RpcViewStateErrorInternalErrorInfo
    name: Literal['INTERNAL_ERROR']

class RpcViewStateError(RootModel[Union[RpcViewStateErrorUnknownBlock, RpcViewStateErrorInvalidAccount, RpcViewStateErrorUnknownAccount, RpcViewStateErrorTooLargeContractState, RpcViewStateErrorInternalError]]):
    pass




"""Version 1 of [`BandwidthRequest`]."""

from near_jsonrpc_models.bandwidth_request import BandwidthRequest
from pydantic import BaseModel
from typing import List


class BandwidthRequestsV1(BaseModel):
    requests: List[BandwidthRequest]



from near_jsonrpc_models.duration_as_std_schema_provider import DurationAsStdSchemaProvider
from pydantic import BaseModel
from pydantic import conint


class EpochSyncConfig(BaseModel):
    # Number of epochs behind the network head beyond which the node will use
    # epoch sync instead of header sync. Also the maximum age (in epochs) of
    # accepted epoch sync proofs. At the consumption site, this is multiplied
    # by epoch_length to get the horizon in blocks.
    epoch_sync_horizon_num_epochs: conint(ge=0, le=18446744073709551615) = 4
    # Timeout for epoch sync requests. The node will continue retrying indefinitely even
    # if this timeout is exceeded.
    timeout_for_epoch_sync: DurationAsStdSchemaProvider = None



from near_jsonrpc_models.strict_model import StrictBaseModel
from near_jsonrpc_models.tx_execution_error import TxExecutionError
from pydantic import BaseModel
from pydantic import RootModel
from typing import Literal
from typing import Union


"""The execution has not yet started."""
class FinalExecutionStatusNotStarted(RootModel[Literal['NotStarted']]):
    pass

"""The execution has started and still going."""
class FinalExecutionStatusStarted(RootModel[Literal['Started']]):
    pass

class FinalExecutionStatusFailure(StrictBaseModel):
    """The execution has failed with the given error."""
    Failure: TxExecutionError

class FinalExecutionStatusSuccessValue(StrictBaseModel):
    """The execution has succeeded and returned some value or an empty vec encoded in base64."""
    SuccessValue: str

class FinalExecutionStatus(RootModel[Union[FinalExecutionStatusNotStarted, FinalExecutionStatusStarted, FinalExecutionStatusFailure, FinalExecutionStatusSuccessValue]]):
    pass




from near_jsonrpc_models.rpc_view_account_request import RpcViewAccountRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForExperimentalViewAccount(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['EXPERIMENTAL_view_account']
    params: RpcViewAccountRequest



from near_jsonrpc_models.rpc_send_transaction_request import RpcSendTransactionRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForBroadcastTxCommit(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['broadcast_tx_commit']
    params: RpcSendTransactionRequest



from near_jsonrpc_models.error_wrapper_for_rpc_view_access_key_error import ErrorWrapperForRpcViewAccessKeyError
from near_jsonrpc_models.rpc_view_access_key_response import RpcViewAccessKeyResponse
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class JsonRpcResponseForRpcViewAccessKeyResponseAndRpcViewAccessKeyErrorResult(BaseModel):
    id: str
    jsonrpc: str
    result: RpcViewAccessKeyResponse

class JsonRpcResponseForRpcViewAccessKeyResponseAndRpcViewAccessKeyErrorError(BaseModel):
    id: str
    jsonrpc: str
    error: ErrorWrapperForRpcViewAccessKeyError

class JsonRpcResponseForRpcViewAccessKeyResponseAndRpcViewAccessKeyError(RootModel[Union[JsonRpcResponseForRpcViewAccessKeyResponseAndRpcViewAccessKeyErrorResult, JsonRpcResponseForRpcViewAccessKeyResponseAndRpcViewAccessKeyErrorError]]):
    pass




"""Supported external storage backends and their minimal config."""

from near_jsonrpc_models.strict_model import StrictBaseModel
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class ExternalStorageLocationS3Payload(BaseModel):
    # Location on S3.
    bucket: str
    # Data may only be available in certain locations.
    region: str

class ExternalStorageLocationS3(StrictBaseModel):
    S3: ExternalStorageLocationS3Payload

class ExternalStorageLocationFilesystemPayload(BaseModel):
    root_dir: str

class ExternalStorageLocationFilesystem(StrictBaseModel):
    """Local filesystem root for storing data."""
    Filesystem: ExternalStorageLocationFilesystemPayload

class ExternalStorageLocationGcsPayload(BaseModel):
    bucket: str

class ExternalStorageLocationGcs(StrictBaseModel):
    """Google Cloud Storage bucket name."""
    GCS: ExternalStorageLocationGcsPayload

class ExternalStorageLocation(RootModel[Union[ExternalStorageLocationS3, ExternalStorageLocationFilesystem, ExternalStorageLocationGcs]]):
    pass




"""An error happened during Action execution"""

from near_jsonrpc_models.action_error_kind import ActionErrorKind
from pydantic import BaseModel
from pydantic import conint


class ActionError(BaseModel):
    # Index of the failed action in the transaction.
    # Action index is not defined if ActionError.kind is `ActionErrorKind::LackBalanceForState`
    index: conint(ge=0, le=18446744073709551615) | None = None
    # The kind of ActionError happened
    kind: ActionErrorKind



from near_jsonrpc_models.error_wrapper_for_rpc_validator_error import ErrorWrapperForRpcValidatorError
from near_jsonrpc_models.validator_stake_view import ValidatorStakeView
from pydantic import BaseModel
from pydantic import RootModel
from typing import List
from typing import Union


class JsonRpcResponseForArrayOfValidatorStakeViewAndRpcValidatorErrorResult(BaseModel):
    id: str
    jsonrpc: str
    result: List[ValidatorStakeView]

class JsonRpcResponseForArrayOfValidatorStakeViewAndRpcValidatorErrorError(BaseModel):
    id: str
    jsonrpc: str
    error: ErrorWrapperForRpcValidatorError

class JsonRpcResponseForArrayOfValidatorStakeViewAndRpcValidatorError(RootModel[Union[JsonRpcResponseForArrayOfValidatorStakeViewAndRpcValidatorErrorResult, JsonRpcResponseForArrayOfValidatorStakeViewAndRpcValidatorErrorError]]):
    pass




"""Errors which may occur during working with trie storages, storing
trie values (trie nodes and state values) by their hashes."""

from near_jsonrpc_models.missing_trie_value import MissingTrieValue
from near_jsonrpc_models.strict_model import StrictBaseModel
from pydantic import BaseModel
from pydantic import RootModel
from typing import Literal
from typing import Union


"""Key-value db internal failure"""
class StorageErrorStorageInternalError(RootModel[Literal['StorageInternalError']]):
    pass

class StorageErrorMissingTrieValue(StrictBaseModel):
    """Requested trie value by its hash which is missing in storage."""
    MissingTrieValue: MissingTrieValue

"""Found trie node which shouldn't be part of state. Raised during
validation of state sync parts where incorrect node was passed.
TODO (#8997): consider including hash of trie node."""
class StorageErrorUnexpectedTrieValue(RootModel[Literal['UnexpectedTrieValue']]):
    pass

class StorageErrorStorageInconsistentState(StrictBaseModel):
    """Either invalid state or key-value db is corrupted.
For PartialStorage it cannot be corrupted.
Error message is unreliable and for debugging purposes only. It's also probably ok to
panic in every place that produces this error.
We can check if db is corrupted by verifying everything in the state trie."""
    StorageInconsistentState: str

class StorageErrorFlatStorageBlockNotSupported(StrictBaseModel):
    """Flat storage error, meaning that it doesn't support some block anymore.
We guarantee that such block cannot become final, thus block processing
must resume normally."""
    FlatStorageBlockNotSupported: str

class StorageErrorMemTrieLoadingError(StrictBaseModel):
    """In-memory trie could not be loaded for some reason."""
    MemTrieLoadingError: str

class StorageError(RootModel[Union[StorageErrorStorageInternalError, StorageErrorMissingTrieValue, StorageErrorUnexpectedTrieValue, StorageErrorStorageInconsistentState, StorageErrorFlatStorageBlockNotSupported, StorageErrorMemTrieLoadingError]]):
    pass




from near_jsonrpc_models.external_storage_location import ExternalStorageLocation
from pydantic import BaseModel
from pydantic import conint


class ExternalStorageConfig(BaseModel):
    # The number of attempts the node will make to obtain a part from peers in
    # the network before it fetches from external storage.
    external_storage_fallback_threshold: conint(ge=0, le=18446744073709551615) = 3
    # Location of state parts.
    location: ExternalStorageLocation = None
    # When fetching state parts from external storage, throttle fetch requests
    # to this many concurrent requests.
    num_concurrent_requests: conint(ge=0, le=255) = 25
    # During catchup, the node will use a different number of concurrent requests
    # to reduce the performance impact of state sync.
    num_concurrent_requests_during_catchup: conint(ge=0, le=255) = 5



from near_jsonrpc_models.crypto_hash import CryptoHash
from pydantic import BaseModel
from pydantic import RootModel
from pydantic import conint
from typing import Union


class BlockIdBlockHeight(RootModel[conint(ge=0, le=18446744073709551615)]):
    pass

class BlockIdCryptoHash(CryptoHash):
    pass

class BlockId(RootModel[Union[BlockIdBlockHeight, BlockIdCryptoHash]]):
    pass




from near_jsonrpc_models.rpc_congestion_level_request import RpcCongestionLevelRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForExperimentalCongestionLevel(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['EXPERIMENTAL_congestion_level']
    params: RpcCongestionLevelRequest



"""A result returned by contract method"""

from near_jsonrpc_models.crypto_hash import CryptoHash
from pydantic import BaseModel
from pydantic import conint
from typing import List


class RpcCallFunctionResponse(BaseModel):
    block_hash: CryptoHash
    block_height: conint(ge=0, le=18446744073709551615)
    logs: List[str]
    result: List[conint(ge=0, le=255)]



from near_jsonrpc_models.error_wrapper_for_rpc_light_client_proof_error import ErrorWrapperForRpcLightClientProofError
from near_jsonrpc_models.rpc_light_client_execution_proof_response import RpcLightClientExecutionProofResponse
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class JsonRpcResponseForRpcLightClientExecutionProofResponseAndRpcLightClientProofErrorResult(BaseModel):
    id: str
    jsonrpc: str
    result: RpcLightClientExecutionProofResponse

class JsonRpcResponseForRpcLightClientExecutionProofResponseAndRpcLightClientProofErrorError(BaseModel):
    id: str
    jsonrpc: str
    error: ErrorWrapperForRpcLightClientProofError

class JsonRpcResponseForRpcLightClientExecutionProofResponseAndRpcLightClientProofError(RootModel[Union[JsonRpcResponseForRpcLightClientExecutionProofResponseAndRpcLightClientProofErrorResult, JsonRpcResponseForRpcLightClientExecutionProofResponseAndRpcLightClientProofErrorError]]):
    pass




from near_jsonrpc_models.error_wrapper_for_rpc_status_error import ErrorWrapperForRpcStatusError
from near_jsonrpc_models.rpc_status_response import RpcStatusResponse
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class JsonRpcResponseForRpcStatusResponseAndRpcStatusErrorResult(BaseModel):
    id: str
    jsonrpc: str
    result: RpcStatusResponse

class JsonRpcResponseForRpcStatusResponseAndRpcStatusErrorError(BaseModel):
    id: str
    jsonrpc: str
    error: ErrorWrapperForRpcStatusError

class JsonRpcResponseForRpcStatusResponseAndRpcStatusError(RootModel[Union[JsonRpcResponseForRpcStatusResponseAndRpcStatusErrorResult, JsonRpcResponseForRpcStatusResponseAndRpcStatusErrorError]]):
    pass




"""NONE: Transaction is waiting to be included into the blockINCLUDED: Transaction is included into the block. The block may be not finalized yetEXECUTED_OPTIMISTIC: Transaction is included into the block +
All non-refund transaction receipts finished their execution.
The corresponding blocks for tx and each receipt may be not finalized yetINCLUDED_FINAL: Transaction is included into finalized blockEXECUTED: Transaction is included into finalized block +
All non-refund transaction receipts finished their execution.
The corresponding blocks for each receipt may be not finalized yetFINAL: Transaction is included into finalized block +
Execution of all transaction receipts is finalized, including refund receipts"""

from pydantic import RootModel
from typing import Literal


class TxExecutionStatus(RootModel[Literal['NONE', 'INCLUDED', 'EXECUTED_OPTIMISTIC', 'INCLUDED_FINAL', 'EXECUTED', 'FINAL']]):
    pass




from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.action_view import ActionView
from near_jsonrpc_models.crypto_hash import CryptoHash
from near_jsonrpc_models.public_key import PublicKey
from near_jsonrpc_models.signature import Signature
from pydantic import BaseModel
from pydantic import conint
from typing import List


class SignedTransactionView(BaseModel):
    actions: List[ActionView]
    hash: CryptoHash
    nonce: conint(ge=0, le=18446744073709551615)
    nonce_index: conint(ge=0, le=65535) | None = None
    # Deprecated, retained for backward compatibility.
    priority_fee: conint(ge=0, le=18446744073709551615) = 0
    public_key: PublicKey
    receiver_id: AccountId
    signature: Signature
    signer_id: AccountId



from near_jsonrpc_models.rpc_validator_request import RpcValidatorRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForValidators(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['validators']
    params: RpcValidatorRequest



from near_jsonrpc_models.global_contract_identifier import GlobalContractIdentifier
from pydantic import BaseModel
from typing import Dict


class DeterministicAccountStateInitV1(BaseModel):
    code: GlobalContractIdentifier
    data: Dict[str, str]



from near_jsonrpc_models.rpc_view_state_request import RpcViewStateRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForExperimentalViewState(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['EXPERIMENTAL_view_state']
    params: RpcViewStateRequest



from near_jsonrpc_models.rpc_chunk_request import RpcChunkRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForChunk(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['chunk']
    params: RpcChunkRequest



from near_jsonrpc_models.crypto_hash import CryptoHash
from pydantic import BaseModel
from pydantic import RootModel
from typing import Literal
from typing import Union


class RpcReceiptErrorInternalErrorInfo(BaseModel):
    error_message: str

class RpcReceiptErrorInternalError(BaseModel):
    info: RpcReceiptErrorInternalErrorInfo
    name: Literal['INTERNAL_ERROR']

class RpcReceiptErrorUnknownReceiptInfo(BaseModel):
    receipt_id: CryptoHash

class RpcReceiptErrorUnknownReceipt(BaseModel):
    info: RpcReceiptErrorUnknownReceiptInfo
    name: Literal['UNKNOWN_RECEIPT']

class RpcReceiptError(RootModel[Union[RpcReceiptErrorInternalError, RpcReceiptErrorUnknownReceipt]]):
    pass




"""The configuration for congestion control. More info about congestion [here](https://near.github.io/nearcore/architecture/how/receipt-congestion.html?highlight=congestion#receipt-congestion)"""

from near_jsonrpc_models.near_gas import NearGas
from pydantic import BaseModel
from pydantic import conint


class CongestionControlConfigView(BaseModel):
    # How much gas the chosen allowed shard can send to a 100% congested shard.
    # 
    # See [`CongestionControlConfig`] for more details.
    allowed_shard_outgoing_gas: NearGas = None
    # How much gas in delayed receipts of a shard is 100% incoming congestion.
    # 
    # See [`CongestionControlConfig`] for more details.
    max_congestion_incoming_gas: NearGas = None
    # How much memory space of all delayed and buffered receipts in a shard is
    # considered 100% congested.
    # 
    # See [`CongestionControlConfig`] for more details.
    max_congestion_memory_consumption: conint(ge=0, le=18446744073709551615) = None
    # How many missed chunks in a row in a shard is considered 100% congested.
    max_congestion_missed_chunks: conint(ge=0, le=18446744073709551615) = None
    # How much gas in outgoing buffered receipts of a shard is 100% congested.
    # 
    # Outgoing congestion contributes to overall congestion, which reduces how
    # much other shards are allowed to forward to this shard.
    max_congestion_outgoing_gas: NearGas = None
    # The maximum amount of gas attached to receipts a shard can forward to
    # another shard per chunk.
    # 
    # See [`CongestionControlConfig`] for more details.
    max_outgoing_gas: NearGas = None
    # The maximum amount of gas in a chunk spent on converting new transactions to
    # receipts.
    # 
    # See [`CongestionControlConfig`] for more details.
    max_tx_gas: NearGas = None
    # The minimum gas each shard can send to a shard that is not fully congested.
    # 
    # See [`CongestionControlConfig`] for more details.
    min_outgoing_gas: NearGas = None
    # The minimum amount of gas in a chunk spent on converting new transactions
    # to receipts, as long as the receiving shard is not congested.
    # 
    # See [`CongestionControlConfig`] for more details.
    min_tx_gas: NearGas = None
    # Large size limit for outgoing receipts to a shard, used when it's safe
    # to send a lot of receipts without making the state witness too large.
    # It limits the total sum of outgoing receipts, not individual receipts.
    outgoing_receipts_big_size_limit: conint(ge=0, le=18446744073709551615) = None
    # The standard size limit for outgoing receipts aimed at a single shard.
    # This limit is pretty small to keep the size of source_receipt_proofs under control.
    # It limits the total sum of outgoing receipts, not individual receipts.
    outgoing_receipts_usual_size_limit: conint(ge=0, le=18446744073709551615) = None
    # How much congestion a shard can tolerate before it stops all shards from
    # accepting new transactions with the receiver set to the congested shard.
    reject_tx_congestion_threshold: float = None



"""Configures whether the node checks the next or the next next epoch for network version compatibility."""

from pydantic import RootModel
from typing import Literal


class ProtocolVersionCheckConfig(RootModel[Literal['Next', 'NextNext']]):
    pass




from near_jsonrpc_models.error_wrapper_for_rpc_view_account_error import ErrorWrapperForRpcViewAccountError
from near_jsonrpc_models.rpc_view_account_response import RpcViewAccountResponse
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class JsonRpcResponseForRpcViewAccountResponseAndRpcViewAccountErrorResult(BaseModel):
    id: str
    jsonrpc: str
    result: RpcViewAccountResponse

class JsonRpcResponseForRpcViewAccountResponseAndRpcViewAccountErrorError(BaseModel):
    id: str
    jsonrpc: str
    error: ErrorWrapperForRpcViewAccountError

class JsonRpcResponseForRpcViewAccountResponseAndRpcViewAccountError(RootModel[Union[JsonRpcResponseForRpcViewAccountResponseAndRpcViewAccountErrorResult, JsonRpcResponseForRpcViewAccountResponseAndRpcViewAccountErrorError]]):
    pass




"""The fees settings for a data receipt creation"""

from near_jsonrpc_models.fee import Fee
from pydantic import BaseModel


class DataReceiptCreationConfigView(BaseModel):
    # Base cost of creating a data receipt.
    # Both `send` and `exec` costs are burned when a new receipt has input dependencies. The gas
    # is charged for each input dependency. The dependencies are specified when a receipt is
    # created using `promise_then` and `promise_batch_then`.
    # NOTE: Any receipt with output dependencies will produce data receipts. Even if it fails.
    # Even if the last action is not a function call (in case of success it will return empty
    # value).
    base_cost: Fee = None
    # Additional cost per byte sent.
    # Both `send` and `exec` costs are burned when a function call finishes execution and returns
    # `N` bytes of data to every output dependency. For each output dependency the cost is
    # `(send(sir) + exec()) * N`.
    cost_per_byte: Fee = None



from near_jsonrpc_models.access_key_view import AccessKeyView
from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.crypto_hash import CryptoHash
from near_jsonrpc_models.delegate_action import DelegateAction
from near_jsonrpc_models.function_args import FunctionArgs
from near_jsonrpc_models.global_contract_identifier_view import GlobalContractIdentifierView
from near_jsonrpc_models.near_gas import NearGas
from near_jsonrpc_models.near_token import NearToken
from near_jsonrpc_models.public_key import PublicKey
from near_jsonrpc_models.signature import Signature
from near_jsonrpc_models.strict_model import StrictBaseModel
from pydantic import BaseModel
from pydantic import RootModel
from typing import Dict
from typing import Literal
from typing import Union


class ActionViewCreateAccount(RootModel[Literal['CreateAccount']]):
    pass

class ActionViewDeployContractPayload(BaseModel):
    code: str

class ActionViewDeployContract(StrictBaseModel):
    DeployContract: ActionViewDeployContractPayload

class ActionViewFunctionCallPayload(BaseModel):
    args: FunctionArgs
    deposit: NearToken
    gas: NearGas
    method_name: str

class ActionViewFunctionCall(StrictBaseModel):
    FunctionCall: ActionViewFunctionCallPayload

class ActionViewTransferPayload(BaseModel):
    deposit: NearToken

class ActionViewTransfer(StrictBaseModel):
    Transfer: ActionViewTransferPayload

class ActionViewStakePayload(BaseModel):
    public_key: PublicKey
    stake: NearToken

class ActionViewStake(StrictBaseModel):
    Stake: ActionViewStakePayload

class ActionViewAddKeyPayload(BaseModel):
    access_key: AccessKeyView
    public_key: PublicKey

class ActionViewAddKey(StrictBaseModel):
    AddKey: ActionViewAddKeyPayload

class ActionViewDeleteKeyPayload(BaseModel):
    public_key: PublicKey

class ActionViewDeleteKey(StrictBaseModel):
    DeleteKey: ActionViewDeleteKeyPayload

class ActionViewDeleteAccountPayload(BaseModel):
    beneficiary_id: AccountId

class ActionViewDeleteAccount(StrictBaseModel):
    DeleteAccount: ActionViewDeleteAccountPayload

class ActionViewDelegatePayload(BaseModel):
    delegate_action: DelegateAction
    signature: Signature

class ActionViewDelegate(StrictBaseModel):
    Delegate: ActionViewDelegatePayload

class ActionViewDeployGlobalContractPayload(BaseModel):
    code: str

class ActionViewDeployGlobalContract(StrictBaseModel):
    DeployGlobalContract: ActionViewDeployGlobalContractPayload

class ActionViewDeployGlobalContractByAccountIdPayload(BaseModel):
    code: str

class ActionViewDeployGlobalContractByAccountId(StrictBaseModel):
    DeployGlobalContractByAccountId: ActionViewDeployGlobalContractByAccountIdPayload

class ActionViewUseGlobalContractPayload(BaseModel):
    code_hash: CryptoHash

class ActionViewUseGlobalContract(StrictBaseModel):
    UseGlobalContract: ActionViewUseGlobalContractPayload

class ActionViewUseGlobalContractByAccountIdPayload(BaseModel):
    account_id: AccountId

class ActionViewUseGlobalContractByAccountId(StrictBaseModel):
    UseGlobalContractByAccountId: ActionViewUseGlobalContractByAccountIdPayload

class ActionViewDeterministicStateInitPayload(BaseModel):
    code: GlobalContractIdentifierView
    data: Dict[str, str]
    deposit: NearToken

class ActionViewDeterministicStateInit(StrictBaseModel):
    DeterministicStateInit: ActionViewDeterministicStateInitPayload

class ActionViewTransferToGasKeyPayload(BaseModel):
    deposit: NearToken
    public_key: PublicKey

class ActionViewTransferToGasKey(StrictBaseModel):
    TransferToGasKey: ActionViewTransferToGasKeyPayload

class ActionViewWithdrawFromGasKeyPayload(BaseModel):
    amount: NearToken
    public_key: PublicKey

class ActionViewWithdrawFromGasKey(StrictBaseModel):
    WithdrawFromGasKey: ActionViewWithdrawFromGasKeyPayload

class ActionView(RootModel[Union[ActionViewCreateAccount, ActionViewDeployContract, ActionViewFunctionCall, ActionViewTransfer, ActionViewStake, ActionViewAddKey, ActionViewDeleteKey, ActionViewDeleteAccount, ActionViewDelegate, ActionViewDeployGlobalContract, ActionViewDeployGlobalContractByAccountId, ActionViewUseGlobalContract, ActionViewUseGlobalContractByAccountId, ActionViewDeterministicStateInit, ActionViewTransferToGasKey, ActionViewWithdrawFromGasKey]]):
    pass




from near_jsonrpc_models.block_id import BlockId
from near_jsonrpc_models.crypto_hash import CryptoHash
from near_jsonrpc_models.shard_id import ShardId
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class RpcCongestionLevelRequestBlockShardId(BaseModel):
    block_id: BlockId
    shard_id: ShardId

class RpcCongestionLevelRequestChunkHash(BaseModel):
    chunk_id: CryptoHash

class RpcCongestionLevelRequest(RootModel[Union[RpcCongestionLevelRequestBlockShardId, RpcCongestionLevelRequestChunkHash]]):
    pass




from near_jsonrpc_models.delegate_action import DelegateAction
from near_jsonrpc_models.signature import Signature
from pydantic import BaseModel


class SignedDelegateAction(BaseModel):
    delegate_action: DelegateAction
    signature: Signature



from near_jsonrpc_models.error_wrapper_for_rpc_validator_error import ErrorWrapperForRpcValidatorError
from near_jsonrpc_models.rpc_validator_response import RpcValidatorResponse
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class JsonRpcResponseForRpcValidatorResponseAndRpcValidatorErrorResult(BaseModel):
    id: str
    jsonrpc: str
    result: RpcValidatorResponse

class JsonRpcResponseForRpcValidatorResponseAndRpcValidatorErrorError(BaseModel):
    id: str
    jsonrpc: str
    error: ErrorWrapperForRpcValidatorError

class JsonRpcResponseForRpcValidatorResponseAndRpcValidatorError(RootModel[Union[JsonRpcResponseForRpcValidatorResponseAndRpcValidatorErrorResult, JsonRpcResponseForRpcValidatorResponseAndRpcValidatorErrorError]]):
    pass




"""Resulting state values for a view state query request"""

from near_jsonrpc_models.crypto_hash import CryptoHash
from near_jsonrpc_models.state_item import StateItem
from pydantic import BaseModel
from pydantic import conint
from typing import List


class RpcViewStateResponse(BaseModel):
    block_hash: CryptoHash
    block_height: conint(ge=0, le=18446744073709551615)
    proof: List[str] = None
    values: List[StateItem]



from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.block_id import BlockId
from near_jsonrpc_models.finality import Finality
from near_jsonrpc_models.sync_checkpoint import SyncCheckpoint
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class RpcViewAccountRequestBlockId(BaseModel):
    account_id: AccountId
    block_id: BlockId

class RpcViewAccountRequestFinality(BaseModel):
    account_id: AccountId
    finality: Finality

class RpcViewAccountRequestSyncCheckpoint(BaseModel):
    account_id: AccountId
    sync_checkpoint: SyncCheckpoint

class RpcViewAccountRequest(RootModel[Union[RpcViewAccountRequestBlockId, RpcViewAccountRequestFinality, RpcViewAccountRequestSyncCheckpoint]]):
    pass




from near_jsonrpc_models.rpc_client_config_request import RpcClientConfigRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForClientConfig(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['client_config']
    params: RpcClientConfigRequest



import httpx


class HttpTransportAsync:
    def __init__(
            self,
            base_url: str,
            *,
            timeout: float = 10.0,
            headers: dict[str, str] | None = None,
    ):
        self._client = httpx.AsyncClient(
            base_url=base_url,
            timeout=timeout,
            headers=headers,
        )

    async def post(self, json: dict) -> httpx.Response:
        return await self._client.post("", json=json)

    async def close(self):
        await self._client.aclose()


class HttpTransportSync:
    def __init__(
            self,
            base_url: str,
            *,
            timeout: float = 10.0,
            headers: dict[str, str] | None = None,
    ):
        self._client = httpx.Client(
            base_url=base_url,
            timeout=timeout,
            headers=headers,
        )

    def post(self, json: dict) -> httpx.Response:
        return self._client.post("", json=json)

    def close(self):
        self._client.close()



from near_jsonrpc_models.signed_transaction import SignedTransaction
from near_jsonrpc_models.tx_execution_status import TxExecutionStatus
from pydantic import BaseModel
from pydantic import Field


class RpcSendTransactionRequest(BaseModel):
    signed_tx_base64: SignedTransaction
    wait_until: TxExecutionStatus = Field(default_factory=lambda: TxExecutionStatus('EXECUTED_OPTIMISTIC'))



from near_jsonrpc_models.block_id import BlockId
from near_jsonrpc_models.finality import Finality
from near_jsonrpc_models.strict_model import StrictBaseModel
from near_jsonrpc_models.sync_checkpoint import SyncCheckpoint
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class BlockReferenceBlockId(StrictBaseModel):
    block_id: BlockId

class BlockReferenceFinality(StrictBaseModel):
    finality: Finality

class BlockReferenceSyncCheckpoint(StrictBaseModel):
    sync_checkpoint: SyncCheckpoint

class BlockReference(RootModel[Union[BlockReferenceBlockId, BlockReferenceFinality, BlockReferenceSyncCheckpoint]]):
    pass




from near_jsonrpc_models.crypto_hash import CryptoHash
from near_jsonrpc_models.execution_outcome_view import ExecutionOutcomeView
from near_jsonrpc_models.merkle_path_item import MerklePathItem
from pydantic import BaseModel
from typing import List


class ExecutionOutcomeWithIdView(BaseModel):
    block_hash: CryptoHash
    id: CryptoHash
    outcome: ExecutionOutcomeView
    proof: List[MerklePathItem]



from near_jsonrpc_models.error_wrapper_for_rpc_block_error import ErrorWrapperForRpcBlockError
from near_jsonrpc_models.rpc_block_response import RpcBlockResponse
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class JsonRpcResponseForRpcBlockResponseAndRpcBlockErrorResult(BaseModel):
    id: str
    jsonrpc: str
    result: RpcBlockResponse

class JsonRpcResponseForRpcBlockResponseAndRpcBlockErrorError(BaseModel):
    id: str
    jsonrpc: str
    error: ErrorWrapperForRpcBlockError

class JsonRpcResponseForRpcBlockResponseAndRpcBlockError(RootModel[Union[JsonRpcResponseForRpcBlockResponseAndRpcBlockErrorResult, JsonRpcResponseForRpcBlockResponseAndRpcBlockErrorError]]):
    pass




from near_jsonrpc_models.rpc_light_client_block_proof_request import RpcLightClientBlockProofRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForExperimentalLightClientBlockProof(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['EXPERIMENTAL_light_client_block_proof']
    params: RpcLightClientBlockProofRequest



from near_jsonrpc_models.rpc_view_code_request import RpcViewCodeRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForExperimentalViewCode(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['EXPERIMENTAL_view_code']
    params: RpcViewCodeRequest



"""This enum represents if a storage_get call will be performed through flat storage or trie"""

from pydantic import RootModel
from typing import Literal


class StorageGetMode(RootModel[Literal['FlatStorage', 'Trie']]):
    pass




from near_jsonrpc_models.rpc_state_changes_in_block_by_type_request import RpcStateChangesInBlockByTypeRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForExperimentalChanges(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['EXPERIMENTAL_changes']
    params: RpcStateChangesInBlockByTypeRequest



from near_jsonrpc_models.internal_error import InternalError
from near_jsonrpc_models.rpc_maintenance_windows_error import RpcMaintenanceWindowsError
from near_jsonrpc_models.rpc_request_validation_error_kind import RpcRequestValidationErrorKind
from pydantic import BaseModel
from pydantic import RootModel
from typing import Literal
from typing import Union


class ErrorWrapperForRpcMaintenanceWindowsErrorRequestValidationError(BaseModel):
    cause: RpcRequestValidationErrorKind
    name: Literal['REQUEST_VALIDATION_ERROR']

class ErrorWrapperForRpcMaintenanceWindowsErrorHandlerError(BaseModel):
    cause: RpcMaintenanceWindowsError
    name: Literal['HANDLER_ERROR']

class ErrorWrapperForRpcMaintenanceWindowsErrorInternalError(BaseModel):
    cause: InternalError
    name: Literal['INTERNAL_ERROR']

class ErrorWrapperForRpcMaintenanceWindowsError(RootModel[Union[ErrorWrapperForRpcMaintenanceWindowsErrorRequestValidationError, ErrorWrapperForRpcMaintenanceWindowsErrorHandlerError, ErrorWrapperForRpcMaintenanceWindowsErrorInternalError]]):
    pass




from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.crypto_hash import CryptoHash
from near_jsonrpc_models.signed_transaction import SignedTransaction
from near_jsonrpc_models.tx_execution_status import TxExecutionStatus
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class RpcTransactionStatusRequestSignedTxBase64(BaseModel):
    wait_until: TxExecutionStatus = 'EXECUTED_OPTIMISTIC'
    signed_tx_base64: SignedTransaction

class RpcTransactionStatusRequestSenderAccountIdTxHash(BaseModel):
    wait_until: TxExecutionStatus = 'EXECUTED_OPTIMISTIC'
    sender_account_id: AccountId
    tx_hash: CryptoHash

class RpcTransactionStatusRequest(RootModel[Union[RpcTransactionStatusRequestSignedTxBase64, RpcTransactionStatusRequestSenderAccountIdTxHash]]):
    pass




from near_jsonrpc_models.block_id import BlockId
from near_jsonrpc_models.epoch_id import EpochId
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class RpcValidatorRequestEpochId(BaseModel):
    epoch_id: EpochId

class RpcValidatorRequestBlockId(BaseModel):
    block_id: BlockId

class RpcValidatorRequestLatest(BaseModel):
    latest: str

class RpcValidatorRequest(RootModel[Union[RpcValidatorRequestEpochId, RpcValidatorRequestBlockId, RpcValidatorRequestLatest]]):
    pass




from near_jsonrpc_models.internal_error import InternalError
from near_jsonrpc_models.rpc_request_validation_error_kind import RpcRequestValidationErrorKind
from near_jsonrpc_models.rpc_status_error import RpcStatusError
from pydantic import BaseModel
from pydantic import RootModel
from typing import Literal
from typing import Union


class ErrorWrapperForRpcStatusErrorRequestValidationError(BaseModel):
    cause: RpcRequestValidationErrorKind
    name: Literal['REQUEST_VALIDATION_ERROR']

class ErrorWrapperForRpcStatusErrorHandlerError(BaseModel):
    cause: RpcStatusError
    name: Literal['HANDLER_ERROR']

class ErrorWrapperForRpcStatusErrorInternalError(BaseModel):
    cause: InternalError
    name: Literal['INTERNAL_ERROR']

class ErrorWrapperForRpcStatusError(RootModel[Union[ErrorWrapperForRpcStatusErrorRequestValidationError, ErrorWrapperForRpcStatusErrorHandlerError, ErrorWrapperForRpcStatusErrorInternalError]]):
    pass




from near_jsonrpc_models.rpc_view_access_key_list_request import RpcViewAccessKeyListRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForExperimentalViewAccessKeyList(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['EXPERIMENTAL_view_access_key_list']
    params: RpcViewAccessKeyListRequest



from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.crypto_hash import CryptoHash
from near_jsonrpc_models.strict_model import StrictBaseModel
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class GlobalContractIdentifierViewHash(StrictBaseModel):
    hash: CryptoHash

class GlobalContractIdentifierViewAccountId(StrictBaseModel):
    account_id: AccountId

class GlobalContractIdentifierView(RootModel[Union[GlobalContractIdentifierViewHash, GlobalContractIdentifierViewAccountId]]):
    pass




"""Defines permissions for AccessKey"""

from near_jsonrpc_models.function_call_permission import FunctionCallPermission
from near_jsonrpc_models.gas_key_info import GasKeyInfo
from near_jsonrpc_models.strict_model import StrictBaseModel
from pydantic import BaseModel
from pydantic import RootModel
from typing import List
from typing import Literal
from typing import Tuple
from typing import Union


class AccessKeyPermissionFunctionCall(StrictBaseModel):
    FunctionCall: FunctionCallPermission

"""Grants full access to the account.
NOTE: It's used to replace account-level public keys."""
class AccessKeyPermissionFullAccess(RootModel[Literal['FullAccess']]):
    pass

class AccessKeyPermissionGasKeyFunctionCall(StrictBaseModel):
    """Gas key with limited permission to make transactions with FunctionCallActions
Gas keys are a kind of access keys with a prepaid balance to pay for gas."""
    GasKeyFunctionCall: Tuple[GasKeyInfo, FunctionCallPermission]

class AccessKeyPermissionGasKeyFullAccess(StrictBaseModel):
    """Gas key with full access to the account.
Gas keys are a kind of access keys with a prepaid balance to pay for gas."""
    GasKeyFullAccess: GasKeyInfo

class AccessKeyPermission(RootModel[Union[AccessKeyPermissionFunctionCall, AccessKeyPermissionFullAccess, AccessKeyPermissionGasKeyFunctionCall, AccessKeyPermissionGasKeyFullAccess]]):
    pass




from near_jsonrpc_models.error_wrapper_for_rpc_state_changes_error import ErrorWrapperForRpcStateChangesError
from near_jsonrpc_models.rpc_state_changes_in_block_by_type_response import RpcStateChangesInBlockByTypeResponse
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class JsonRpcResponseForRpcStateChangesInBlockByTypeResponseAndRpcStateChangesErrorResult(BaseModel):
    id: str
    jsonrpc: str
    result: RpcStateChangesInBlockByTypeResponse

class JsonRpcResponseForRpcStateChangesInBlockByTypeResponseAndRpcStateChangesErrorError(BaseModel):
    id: str
    jsonrpc: str
    error: ErrorWrapperForRpcStateChangesError

class JsonRpcResponseForRpcStateChangesInBlockByTypeResponseAndRpcStateChangesError(RootModel[Union[JsonRpcResponseForRpcStateChangesInBlockByTypeResponseAndRpcStateChangesErrorResult, JsonRpcResponseForRpcStateChangesInBlockByTypeResponseAndRpcStateChangesErrorError]]):
    pass




"""Reasons for removing a validator from the validator set."""

from near_jsonrpc_models.near_token import NearToken
from near_jsonrpc_models.strict_model import StrictBaseModel
from pydantic import BaseModel
from pydantic import RootModel
from pydantic import conint
from typing import Literal
from typing import Union


"""Deprecated"""
class ValidatorKickoutReasonUnusedSlashed(RootModel[Literal['_UnusedSlashed']]):
    pass

class ValidatorKickoutReasonNotEnoughBlocksPayload(BaseModel):
    expected: conint(ge=0, le=18446744073709551615)
    produced: conint(ge=0, le=18446744073709551615)

class ValidatorKickoutReasonNotEnoughBlocks(StrictBaseModel):
    """Validator didn't produce enough blocks."""
    NotEnoughBlocks: ValidatorKickoutReasonNotEnoughBlocksPayload

class ValidatorKickoutReasonNotEnoughChunksPayload(BaseModel):
    expected: conint(ge=0, le=18446744073709551615)
    produced: conint(ge=0, le=18446744073709551615)

class ValidatorKickoutReasonNotEnoughChunks(StrictBaseModel):
    """Validator didn't produce enough chunks."""
    NotEnoughChunks: ValidatorKickoutReasonNotEnoughChunksPayload

"""Validator unstaked themselves."""
class ValidatorKickoutReasonUnstaked(RootModel[Literal['Unstaked']]):
    pass

class ValidatorKickoutReasonNotEnoughStakePayload(BaseModel):
    stake_u128: NearToken
    threshold_u128: NearToken

class ValidatorKickoutReasonNotEnoughStake(StrictBaseModel):
    """Validator stake is now below threshold"""
    NotEnoughStake: ValidatorKickoutReasonNotEnoughStakePayload

"""Enough stake but is not chosen because of seat limits."""
class ValidatorKickoutReasonDidNotGetASeat(RootModel[Literal['DidNotGetASeat']]):
    pass

class ValidatorKickoutReasonNotEnoughChunkEndorsementsPayload(BaseModel):
    expected: conint(ge=0, le=18446744073709551615)
    produced: conint(ge=0, le=18446744073709551615)

class ValidatorKickoutReasonNotEnoughChunkEndorsements(StrictBaseModel):
    """Validator didn't produce enough chunk endorsements."""
    NotEnoughChunkEndorsements: ValidatorKickoutReasonNotEnoughChunkEndorsementsPayload

class ValidatorKickoutReasonProtocolVersionTooOldPayload(BaseModel):
    network_version: conint(ge=0, le=4294967295)
    version: conint(ge=0, le=4294967295)

class ValidatorKickoutReasonProtocolVersionTooOld(StrictBaseModel):
    """Validator's last block proposal was for a protocol version older than
the network's voted protocol version."""
    ProtocolVersionTooOld: ValidatorKickoutReasonProtocolVersionTooOldPayload

class ValidatorKickoutReason(RootModel[Union[ValidatorKickoutReasonUnusedSlashed, ValidatorKickoutReasonNotEnoughBlocks, ValidatorKickoutReasonNotEnoughChunks, ValidatorKickoutReasonUnstaked, ValidatorKickoutReasonNotEnoughStake, ValidatorKickoutReasonDidNotGetASeat, ValidatorKickoutReasonNotEnoughChunkEndorsements, ValidatorKickoutReasonProtocolVersionTooOld]]):
    pass




from .client import NearClientAsync, NearClientSync
from .transport import HttpTransportAsync, HttpTransportSync
from .errors import (
    RpcClientError,
    RpcTransportError,
    RpcHttpError,
    RpcError,
    RpcTimeoutError
)

__all__ = [
    # Main clients
    "NearClientAsync",
    "NearClientSync",

    # Transport

    "HttpTransportAsync",
    "HttpTransportSync",

    # Errors
    "RpcClientError",
    "RpcTransportError",
    "RpcHttpError",
    "RpcError",
    "RpcTimeoutError"
]



"""Status of the [catchup](https://near.github.io/nearcore/architecture/how/sync.html#catchup) process"""

from near_jsonrpc_models.block_status_view import BlockStatusView
from near_jsonrpc_models.crypto_hash import CryptoHash
from pydantic import BaseModel
from pydantic import conint
from pydantic import field_validator
from typing import Dict
from typing import List


class CatchupStatusView(BaseModel):
    blocks_to_catchup: List[BlockStatusView]
    shard_sync_status: Dict[str, str]
    sync_block_hash: CryptoHash
    sync_block_height: conint(ge=0, le=18446744073709551615)

    @field_validator('shard_sync_status')
    def validate_shard_sync_status_keys(cls, v):
        import re
        pattern = re.compile(r"^\d+$")
        if not isinstance(v, dict):
            raise TypeError('shard_sync_status must be a dict')
        for key in v.keys():
            if not pattern.match(key):
                raise ValueError(f"Invalid key '{key}' in shard_sync_status. Must match '^\\d+$'")
        return v




from near_jsonrpc_models.error_wrapper_for_rpc_chunk_error import ErrorWrapperForRpcChunkError
from near_jsonrpc_models.rpc_chunk_response import RpcChunkResponse
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class JsonRpcResponseForRpcChunkResponseAndRpcChunkErrorResult(BaseModel):
    id: str
    jsonrpc: str
    result: RpcChunkResponse

class JsonRpcResponseForRpcChunkResponseAndRpcChunkErrorError(BaseModel):
    id: str
    jsonrpc: str
    error: ErrorWrapperForRpcChunkError

class JsonRpcResponseForRpcChunkResponseAndRpcChunkError(RootModel[Union[JsonRpcResponseForRpcChunkResponseAndRpcChunkErrorResult, JsonRpcResponseForRpcChunkResponseAndRpcChunkErrorError]]):
    pass




from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.crypto_hash import CryptoHash
from near_jsonrpc_models.detailed_debug_status import DetailedDebugStatus
from near_jsonrpc_models.public_key import PublicKey
from near_jsonrpc_models.status_sync_info import StatusSyncInfo
from near_jsonrpc_models.validator_info import ValidatorInfo
from near_jsonrpc_models.version import Version
from pydantic import BaseModel
from pydantic import conint
from typing import List


class RpcStatusResponse(BaseModel):
    # Unique chain id.
    chain_id: str
    # Information about last blocks, network, epoch and chain & chunk info.
    detailed_debug_status: DetailedDebugStatus | None = None
    # Genesis hash of the chain.
    genesis_hash: CryptoHash
    # Latest protocol version that this client supports.
    latest_protocol_version: conint(ge=0, le=4294967295)
    # Deprecated; same as `validator_public_key` which you should use instead.
    node_key: PublicKey | None = None
    # Public key of the node.
    node_public_key: PublicKey
    # Currently active protocol version.
    protocol_version: conint(ge=0, le=4294967295)
    # Address for RPC server.  None if node doesn't have RPC endpoint enabled.
    rpc_addr: str | None = None
    # Sync status of the node.
    sync_info: StatusSyncInfo
    # Uptime of the node.
    uptime_sec: int
    # Validator id of the node
    validator_account_id: AccountId | None = None
    # Public key of the validator.
    validator_public_key: PublicKey | None = None
    # Current epoch validators.
    validators: List[ValidatorInfo]
    # Binary version.
    version: Version



from near_jsonrpc_models.rpc_status_request import RpcStatusRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForStatus(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['status']
    params: RpcStatusRequest



"""Use global contract action"""

from near_jsonrpc_models.global_contract_identifier import GlobalContractIdentifier
from pydantic import BaseModel


class UseGlobalContractAction(BaseModel):
    contract_identifier: GlobalContractIdentifier



from near_jsonrpc_models.rpc_light_client_execution_proof_request import RpcLightClientExecutionProofRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForExperimentalLightClientProof(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['EXPERIMENTAL_light_client_proof']
    params: RpcLightClientExecutionProofRequest



from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.block_id import BlockId
from near_jsonrpc_models.finality import Finality
from near_jsonrpc_models.store_key import StoreKey
from near_jsonrpc_models.sync_checkpoint import SyncCheckpoint
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class RpcViewStateRequestBlockId(BaseModel):
    account_id: AccountId
    include_proof: bool = False
    prefix_base64: StoreKey
    block_id: BlockId

class RpcViewStateRequestFinality(BaseModel):
    account_id: AccountId
    include_proof: bool = False
    prefix_base64: StoreKey
    finality: Finality

class RpcViewStateRequestSyncCheckpoint(BaseModel):
    account_id: AccountId
    include_proof: bool = False
    prefix_base64: StoreKey
    sync_checkpoint: SyncCheckpoint

class RpcViewStateRequest(RootModel[Union[RpcViewStateRequestBlockId, RpcViewStateRequestFinality, RpcViewStateRequestSyncCheckpoint]]):
    pass




from near_jsonrpc_models.error_wrapper_for_rpc_maintenance_windows_error import ErrorWrapperForRpcMaintenanceWindowsError
from near_jsonrpc_models.range_of_uint64 import RangeOfUint64
from pydantic import BaseModel
from pydantic import RootModel
from typing import List
from typing import Union


class JsonRpcResponseForArrayOfRangeOfUint64AndRpcMaintenanceWindowsErrorResult(BaseModel):
    id: str
    jsonrpc: str
    result: List[RangeOfUint64]

class JsonRpcResponseForArrayOfRangeOfUint64AndRpcMaintenanceWindowsErrorError(BaseModel):
    id: str
    jsonrpc: str
    error: ErrorWrapperForRpcMaintenanceWindowsError

class JsonRpcResponseForArrayOfRangeOfUint64AndRpcMaintenanceWindowsError(RootModel[Union[JsonRpcResponseForArrayOfRangeOfUint64AndRpcMaintenanceWindowsErrorResult, JsonRpcResponseForArrayOfRangeOfUint64AndRpcMaintenanceWindowsErrorError]]):
    pass




from near_jsonrpc_models.error_wrapper_for_rpc_query_error import ErrorWrapperForRpcQueryError
from near_jsonrpc_models.rpc_query_response import RpcQueryResponse
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class JsonRpcResponseForRpcQueryResponseAndRpcQueryErrorResult(BaseModel):
    id: str
    jsonrpc: str
    result: RpcQueryResponse

class JsonRpcResponseForRpcQueryResponseAndRpcQueryErrorError(BaseModel):
    id: str
    jsonrpc: str
    error: ErrorWrapperForRpcQueryError

class JsonRpcResponseForRpcQueryResponseAndRpcQueryError(RootModel[Union[JsonRpcResponseForRpcQueryResponseAndRpcQueryErrorResult, JsonRpcResponseForRpcQueryResponseAndRpcQueryErrorError]]):
    pass




from pydantic import BaseModel
from pydantic import RootModel
from typing import Literal
from typing import Union


class RpcMaintenanceWindowsErrorInternalErrorInfo(BaseModel):
    error_message: str

class RpcMaintenanceWindowsErrorInternalError(BaseModel):
    info: RpcMaintenanceWindowsErrorInternalErrorInfo
    name: Literal['INTERNAL_ERROR']

class RpcMaintenanceWindowsError(RootModel[Union[RpcMaintenanceWindowsErrorInternalError]]):
    pass




from near_jsonrpc_models.crypto_hash import CryptoHash
from near_jsonrpc_models.error_wrapper_for_rpc_transaction_error import ErrorWrapperForRpcTransactionError
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class JsonRpcResponseForCryptoHashAndRpcTransactionErrorResult(BaseModel):
    id: str
    jsonrpc: str
    result: CryptoHash

class JsonRpcResponseForCryptoHashAndRpcTransactionErrorError(BaseModel):
    id: str
    jsonrpc: str
    error: ErrorWrapperForRpcTransactionError

class JsonRpcResponseForCryptoHashAndRpcTransactionError(RootModel[Union[JsonRpcResponseForCryptoHashAndRpcTransactionErrorResult, JsonRpcResponseForCryptoHashAndRpcTransactionErrorError]]):
    pass




from near_jsonrpc_models.rpc_validators_ordered_request import RpcValidatorsOrderedRequest
from pydantic import BaseModel
from typing import Literal


class JsonRpcRequestForExperimentalValidatorsOrdered(BaseModel):
    id: str
    jsonrpc: str
    method: Literal['EXPERIMENTAL_validators_ordered']
    params: RpcValidatorsOrderedRequest



from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.block_reference import BlockReference
from near_jsonrpc_models.crypto_hash import CryptoHash
from pydantic import BaseModel
from pydantic import RootModel
from pydantic import conint
from typing import Literal
from typing import Union


class RpcViewGasKeyListErrorUnknownBlockInfo(BaseModel):
    block_reference: BlockReference

class RpcViewGasKeyListErrorUnknownBlock(BaseModel):
    info: RpcViewGasKeyListErrorUnknownBlockInfo
    name: Literal['UNKNOWN_BLOCK']

class RpcViewGasKeyListErrorInvalidAccountInfo(BaseModel):
    block_hash: CryptoHash
    block_height: conint(ge=0, le=18446744073709551615)
    requested_account_id: AccountId

class RpcViewGasKeyListErrorInvalidAccount(BaseModel):
    info: RpcViewGasKeyListErrorInvalidAccountInfo
    name: Literal['INVALID_ACCOUNT']

class RpcViewGasKeyListErrorUnknownAccountInfo(BaseModel):
    block_hash: CryptoHash
    block_height: conint(ge=0, le=18446744073709551615)
    requested_account_id: AccountId

class RpcViewGasKeyListErrorUnknownAccount(BaseModel):
    info: RpcViewGasKeyListErrorUnknownAccountInfo
    name: Literal['UNKNOWN_ACCOUNT']

class RpcViewGasKeyListErrorInternalErrorInfo(BaseModel):
    error_message: str

class RpcViewGasKeyListErrorInternalError(BaseModel):
    info: RpcViewGasKeyListErrorInternalErrorInfo
    name: Literal['INTERNAL_ERROR']

class RpcViewGasKeyListError(RootModel[Union[RpcViewGasKeyListErrorUnknownBlock, RpcViewGasKeyListErrorInvalidAccount, RpcViewGasKeyListErrorUnknownAccount, RpcViewGasKeyListErrorInternalError]]):
    pass




from near_jsonrpc_models.error_wrapper_for_rpc_view_code_error import ErrorWrapperForRpcViewCodeError
from near_jsonrpc_models.rpc_view_code_response import RpcViewCodeResponse
from pydantic import BaseModel
from pydantic import RootModel
from typing import Union


class JsonRpcResponseForRpcViewCodeResponseAndRpcViewCodeErrorResult(BaseModel):
    id: str
    jsonrpc: str
    result: RpcViewCodeResponse

class JsonRpcResponseForRpcViewCodeResponseAndRpcViewCodeErrorError(BaseModel):
    id: str
    jsonrpc: str
    error: ErrorWrapperForRpcViewCodeError

class JsonRpcResponseForRpcViewCodeResponseAndRpcViewCodeError(RootModel[Union[JsonRpcResponseForRpcViewCodeResponseAndRpcViewCodeErrorResult, JsonRpcResponseForRpcViewCodeResponseAndRpcViewCodeErrorError]]):
    pass




from near_jsonrpc_models.internal_error import InternalError
from near_jsonrpc_models.rpc_protocol_config_error import RpcProtocolConfigError
from near_jsonrpc_models.rpc_request_validation_error_kind import RpcRequestValidationErrorKind
from pydantic import BaseModel
from pydantic import RootModel
from typing import Literal
from typing import Union


class ErrorWrapperForRpcProtocolConfigErrorRequestValidationError(BaseModel):
    cause: RpcRequestValidationErrorKind
    name: Literal['REQUEST_VALIDATION_ERROR']

class ErrorWrapperForRpcProtocolConfigErrorHandlerError(BaseModel):
    cause: RpcProtocolConfigError
    name: Literal['HANDLER_ERROR']

class ErrorWrapperForRpcProtocolConfigErrorInternalError(BaseModel):
    cause: InternalError
    name: Literal['INTERNAL_ERROR']

class ErrorWrapperForRpcProtocolConfigError(RootModel[Union[ErrorWrapperForRpcProtocolConfigErrorRequestValidationError, ErrorWrapperForRpcProtocolConfigErrorHandlerError, ErrorWrapperForRpcProtocolConfigErrorInternalError]]):
    pass




from pydantic import BaseModel
from pydantic import RootModel
from typing import Literal
from typing import Union


class RpcClientConfigErrorInternalErrorInfo(BaseModel):
    error_message: str

class RpcClientConfigErrorInternalError(BaseModel):
    info: RpcClientConfigErrorInternalErrorInfo
    name: Literal['INTERNAL_ERROR']

class RpcClientConfigError(RootModel[Union[RpcClientConfigErrorInternalError]]):
    pass




from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.block_reference import BlockReference
from near_jsonrpc_models.crypto_hash import CryptoHash
from near_jsonrpc_models.function_call_error import FunctionCallError
from pydantic import BaseModel
from pydantic import RootModel
from pydantic import conint
from typing import Literal
from typing import Union


class RpcCallFunctionErrorUnknownBlockInfo(BaseModel):
    block_reference: BlockReference

class RpcCallFunctionErrorUnknownBlock(BaseModel):
    info: RpcCallFunctionErrorUnknownBlockInfo
    name: Literal['UNKNOWN_BLOCK']

class RpcCallFunctionErrorInvalidAccountInfo(BaseModel):
    block_hash: CryptoHash
    block_height: conint(ge=0, le=18446744073709551615)
    requested_account_id: AccountId

class RpcCallFunctionErrorInvalidAccount(BaseModel):
    info: RpcCallFunctionErrorInvalidAccountInfo
    name: Literal['INVALID_ACCOUNT']

class RpcCallFunctionErrorUnknownAccountInfo(BaseModel):
    block_hash: CryptoHash
    block_height: conint(ge=0, le=18446744073709551615)
    requested_account_id: AccountId

class RpcCallFunctionErrorUnknownAccount(BaseModel):
    info: RpcCallFunctionErrorUnknownAccountInfo
    name: Literal['UNKNOWN_ACCOUNT']

class RpcCallFunctionErrorNoContractCodeInfo(BaseModel):
    block_hash: CryptoHash
    block_height: conint(ge=0, le=18446744073709551615)
    contract_account_id: AccountId

class RpcCallFunctionErrorNoContractCode(BaseModel):
    info: RpcCallFunctionErrorNoContractCodeInfo
    name: Literal['NO_CONTRACT_CODE']

class RpcCallFunctionErrorContractExecutionErrorInfo(BaseModel):
    block_hash: CryptoHash
    block_height: conint(ge=0, le=18446744073709551615)
    vm_error: FunctionCallError

class RpcCallFunctionErrorContractExecutionError(BaseModel):
    info: RpcCallFunctionErrorContractExecutionErrorInfo
    name: Literal['CONTRACT_EXECUTION_ERROR']

class RpcCallFunctionErrorInternalErrorInfo(BaseModel):
    error_message: str

class RpcCallFunctionErrorInternalError(BaseModel):
    info: RpcCallFunctionErrorInternalErrorInfo
    name: Literal['INTERNAL_ERROR']

class RpcCallFunctionError(RootModel[Union[RpcCallFunctionErrorUnknownBlock, RpcCallFunctionErrorInvalidAccount, RpcCallFunctionErrorUnknownAccount, RpcCallFunctionErrorNoContractCode, RpcCallFunctionErrorContractExecutionError, RpcCallFunctionErrorInternalError]]):
    pass




from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.peer_id import PeerId
from pydantic import BaseModel


class RpcKnownProducer(BaseModel):
    account_id: AccountId
    addr: str | None = None
    peer_id: PeerId



from near_jsonrpc_models.crypto_hash import CryptoHash
from near_jsonrpc_models.state_change_with_cause_view import StateChangeWithCauseView
from pydantic import BaseModel
from typing import List


class RpcStateChangesInBlockResponse(BaseModel):
    block_hash: CryptoHash
    changes: List[StateChangeWithCauseView]



from datetime import datetime
from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.account_info import AccountInfo
from near_jsonrpc_models.near_gas import NearGas
from near_jsonrpc_models.near_token import NearToken
from near_jsonrpc_models.shard_layout import ShardLayout
from pydantic import BaseModel
from pydantic import Field
from pydantic import conint
from pydantic import conlist
from pydantic import field_validator
from typing import List


class GenesisConfig(BaseModel):
    # Expected number of hidden validators per shard.
    avg_hidden_validator_seats_per_shard: List[conint(ge=0, le=18446744073709551615)]
    # Threshold for kicking out block producers, between 0 and 100.
    block_producer_kickout_threshold: conint(ge=0, le=255)
    # ID of the blockchain. This must be unique for every blockchain.
    # If your testnet blockchains do not have unique chain IDs, you will have a bad time.
    chain_id: str
    # Limits the number of shard changes in chunk producer assignments,
    # if algorithm is able to choose assignment with better balance of
    # number of chunk producers for shards.
    chunk_producer_assignment_changes_limit: conint(ge=0, le=18446744073709551615) = 5
    # Threshold for kicking out chunk producers, between 0 and 100.
    chunk_producer_kickout_threshold: conint(ge=0, le=255)
    # Threshold for kicking out nodes which are only chunk validators, between 0 and 100.
    chunk_validator_only_kickout_threshold: conint(ge=0, le=255) = 80
    # Enable dynamic re-sharding.
    dynamic_resharding: bool
    # Epoch length counted in block heights.
    epoch_length: conint(ge=0, le=18446744073709551615)
    # Fishermen stake threshold.
    fishermen_threshold: NearToken
    # Initial gas limit.
    gas_limit: NearGas
    # Gas price adjustment rate
    gas_price_adjustment_rate: conlist(conint(ge=-2147483648, le=2147483647), min_length=2, max_length=2)
    # Height of genesis block.
    genesis_height: conint(ge=0, le=18446744073709551615)
    # Official time of blockchain start.
    genesis_time: datetime
    max_gas_price: NearToken
    # Maximum inflation on the total supply every epoch.
    max_inflation_rate: conlist(conint(ge=-2147483648, le=2147483647), min_length=2, max_length=2)
    # Max stake percentage of the validators we will kick out.
    max_kickout_stake_perc: conint(ge=0, le=255) = 100
    # Minimum gas price. It is also the initial gas price.
    min_gas_price: NearToken
    # The minimum stake required for staking is last seat price divided by this number.
    minimum_stake_divisor: conint(ge=0, le=18446744073709551615) = 10
    # The lowest ratio s/s_total any block producer can have.
    # See <https://github.com/near/NEPs/pull/167> for details
    minimum_stake_ratio: conlist(conint(ge=-2147483648, le=2147483647), min_length=2, max_length=2) = Field(default_factory=lambda: [1, 6250])
    # The minimum number of validators each shard must have
    minimum_validators_per_shard: conint(ge=0, le=18446744073709551615) = 1
    # Number of block producer seats at genesis.
    num_block_producer_seats: conint(ge=0, le=18446744073709551615)
    # Defines number of shards and number of block producer seats per each shard at genesis.
    # Note: not used with protocol_feature_chunk_only_producers -- replaced by minimum_validators_per_shard
    # Note: not used before as all block producers produce chunks for all shards
    num_block_producer_seats_per_shard: List[conint(ge=0, le=18446744073709551615)]
    # Expected number of blocks per year
    num_blocks_per_year: conint(ge=0, le=18446744073709551615)
    # Deprecated.
    num_chunk_only_producer_seats: conint(ge=0, le=18446744073709551615) = 300
    # Number of chunk producers.
    # Don't mess it up with chunk-only producers feature which is deprecated.
    num_chunk_producer_seats: conint(ge=0, le=18446744073709551615) = 100
    num_chunk_validator_seats: conint(ge=0, le=18446744073709551615) = 300
    # Online maximum threshold above which validator gets full reward.
    online_max_threshold: conlist(conint(ge=-2147483648, le=2147483647), min_length=2, max_length=2) = Field(default_factory=lambda: [99, 100])
    # Online minimum threshold below which validator doesn't receive reward.
    online_min_threshold: conlist(conint(ge=-2147483648, le=2147483647), min_length=2, max_length=2) = Field(default_factory=lambda: [9, 10])
    # Protocol treasury rate
    protocol_reward_rate: conlist(conint(ge=-2147483648, le=2147483647), min_length=2, max_length=2)
    # Protocol treasury account
    protocol_treasury_account: AccountId
    # Threshold of stake that needs to indicate that they ready for upgrade.
    protocol_upgrade_stake_threshold: conlist(conint(ge=-2147483648, le=2147483647), min_length=2, max_length=2) = Field(default_factory=lambda: [4, 5])
    # Protocol version that this genesis works with.
    protocol_version: conint(ge=0, le=4294967295)
    # Layout information regarding how to split accounts to shards
    shard_layout: ShardLayout = Field(default_factory=lambda: ShardLayout(**{'V2': {'boundary_accounts': [], 'id_to_index_map': {'0': 0}, 'index_to_id_map': {'0': 0}, 'shard_ids': [0], 'version': 0}}))
    # If true, shuffle the chunk producers across shards. In other words, if
    # the shard assignments were `[S_0, S_1, S_2, S_3]` where `S_i` represents
    # the set of chunk producers for shard `i`, if this flag were true, the
    # shard assignments might become, for example, `[S_2, S_0, S_3, S_1]`.
    shuffle_shard_assignment_for_chunk_producers: bool = False
    # Number of target chunk validator mandates for each shard.
    target_validator_mandates_per_shard: conint(ge=0, le=18446744073709551615) = 68
    # Total supply of tokens at genesis.
    total_supply: NearToken
    # Number of blocks for which a given transaction is valid
    transaction_validity_period: conint(ge=0, le=18446744073709551615)
    # This is only for test purposes. We hard code some configs for mainnet and testnet
    # in AllEpochConfig, and we want to have a way to test that code path. This flag is for that.
    # If set to true, the node will use the same config override path as mainnet and testnet.
    use_production_config: bool = False
    # List of initial validators.
    validators: List[AccountInfo]

    @field_validator('genesis_time', mode='before')
    def parse_genesis_time_to_datetime(cls, v):
        from datetime import datetime
        if v is None:
            return v
        if isinstance(v, datetime):
            return v
        if isinstance(v, str):
            s = v
            if s.endswith('Z'):
                s = s[:-1] + '+00:00'
            try:
                return datetime.fromisoformat(s)
            except Exception as e:
                raise ValueError(f"genesis_time must be an ISO-8601 datetime string: {e}")
        raise TypeError('genesis_time must be a datetime or ISO-8601 string')




from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.near_token import NearToken
from near_jsonrpc_models.public_key import PublicKey
from pydantic import BaseModel


class ValidatorStakeViewV1(BaseModel):
    account_id: AccountId
    public_key: PublicKey
    stake: NearToken



from near_jsonrpc_models.dump_config import DumpConfig
from near_jsonrpc_models.sync_concurrency import SyncConcurrency
from near_jsonrpc_models.sync_config import SyncConfig
from pydantic import BaseModel
from pydantic import conint


class StateSyncConfig(BaseModel):
    concurrency: SyncConcurrency = None
    # `none` value disables state dump to external storage.
    dump: DumpConfig | None = None
    # Zstd compression level for state parts.
    parts_compression_lvl: conint(ge=-2147483648, le=2147483647) = 1
    sync: SyncConfig = None



"""Reason why a gas key transaction failed at the deposit/account level.
In these cases, gas is still charged from the gas key."""

from pydantic import RootModel
from typing import Literal


class DepositCostFailureReason(RootModel[Literal['NotEnoughBalance', 'LackBalanceForState']]):
    pass




from near_jsonrpc_models.internal_error import InternalError
from near_jsonrpc_models.rpc_gas_price_error import RpcGasPriceError
from near_jsonrpc_models.rpc_request_validation_error_kind import RpcRequestValidationErrorKind
from pydantic import BaseModel
from pydantic import RootModel
from typing import Literal
from typing import Union


class ErrorWrapperForRpcGasPriceErrorRequestValidationError(BaseModel):
    cause: RpcRequestValidationErrorKind
    name: Literal['REQUEST_VALIDATION_ERROR']

class ErrorWrapperForRpcGasPriceErrorHandlerError(BaseModel):
    cause: RpcGasPriceError
    name: Literal['HANDLER_ERROR']

class ErrorWrapperForRpcGasPriceErrorInternalError(BaseModel):
    cause: InternalError
    name: Literal['INTERNAL_ERROR']

class ErrorWrapperForRpcGasPriceError(RootModel[Union[ErrorWrapperForRpcGasPriceErrorRequestValidationError, ErrorWrapperForRpcGasPriceErrorHandlerError, ErrorWrapperForRpcGasPriceErrorInternalError]]):
    pass




"""An action which stakes signer_id tokens and setup's validator public key"""

from near_jsonrpc_models.near_token import NearToken
from near_jsonrpc_models.public_key import PublicKey
from pydantic import BaseModel


class StakeAction(BaseModel):
    # Validator key which will be used to sign transactions on behalf of signer_id
    public_key: PublicKey
    # Amount of tokens to stake.
    stake: NearToken



from near_jsonrpc_models.crypto_hash import CryptoHash
from pydantic import BaseModel
from pydantic import RootModel
from typing import Any
from typing import Dict
from typing import Literal
from typing import Union


class RpcTransactionErrorInvalidTransaction(BaseModel):
    info: Dict[str, Any]
    name: Literal['INVALID_TRANSACTION']

class RpcTransactionErrorDoesNotTrackShard(BaseModel):
    name: Literal['DOES_NOT_TRACK_SHARD']

class RpcTransactionErrorRequestRoutedInfo(BaseModel):
    transaction_hash: CryptoHash

class RpcTransactionErrorRequestRouted(BaseModel):
    info: RpcTransactionErrorRequestRoutedInfo
    name: Literal['REQUEST_ROUTED']

class RpcTransactionErrorUnknownTransactionInfo(BaseModel):
    requested_transaction_hash: CryptoHash

class RpcTransactionErrorUnknownTransaction(BaseModel):
    info: RpcTransactionErrorUnknownTransactionInfo
    name: Literal['UNKNOWN_TRANSACTION']

class RpcTransactionErrorInternalErrorInfo(BaseModel):
    debug_info: str

class RpcTransactionErrorInternalError(BaseModel):
    info: RpcTransactionErrorInternalErrorInfo
    name: Literal['INTERNAL_ERROR']

class RpcTransactionErrorTimeoutError(BaseModel):
    name: Literal['TIMEOUT_ERROR']

class RpcTransactionError(RootModel[Union[RpcTransactionErrorInvalidTransaction, RpcTransactionErrorDoesNotTrackShard, RpcTransactionErrorRequestRouted, RpcTransactionErrorUnknownTransaction, RpcTransactionErrorInternalError, RpcTransactionErrorTimeoutError]]):
    pass




from near_jsonrpc_models.deterministic_account_state_init import DeterministicAccountStateInit
from near_jsonrpc_models.near_token import NearToken
from pydantic import BaseModel


class DeterministicStateInitAction(BaseModel):
    deposit: NearToken
    state_init: DeterministicAccountStateInit



"""Configures how to fetch state parts during state sync."""

from near_jsonrpc_models.external_storage_config import ExternalStorageConfig
from near_jsonrpc_models.strict_model import StrictBaseModel
from pydantic import BaseModel
from pydantic import RootModel
from typing import Literal
from typing import Union


"""Syncs state from the peers without reading anything from external storage."""
class SyncConfigPeers(RootModel[Literal['Peers']]):
    pass

class SyncConfigExternalStorage(StrictBaseModel):
    """Expects parts to be available in external storage.

Usually as a fallback after some number of attempts to use peers."""
    ExternalStorage: ExternalStorageConfig

class SyncConfig(RootModel[Union[SyncConfigPeers, SyncConfigExternalStorage]]):
    pass




from near_jsonrpc_models.account_id import AccountId
from near_jsonrpc_models.crypto_hash import CryptoHash
from near_jsonrpc_models.public_key import PublicKey
from near_jsonrpc_models.shard_id import ShardId
from pydantic import BaseModel
from pydantic import conint
from typing import List


class PeerInfoView(BaseModel):
    account_id: AccountId | None = None
    addr: str
    archival: bool
    block_hash: CryptoHash | None = None
    connection_established_time_millis: conint(ge=0, le=18446744073709551615)
    height: conint(ge=0, le=18446744073709551615) | None = None
    is_highest_block_invalid: bool
    is_outbound_peer: bool
    last_time_peer_requested_millis: conint(ge=0, le=18446744073709551615)
    last_time_received_message_millis: conint(ge=0, le=18446744073709551615)
    # Connection nonce.
    nonce: conint(ge=0, le=18446744073709551615)
    peer_id: PublicKey
    received_bytes_per_sec: conint(ge=0, le=18446744073709551615)
    sent_bytes_per_sec: conint(ge=0, le=18446744073709551615)
    tracked_shards: List[ShardId]
